/*
 Copyright (C) 2007 by Ben Anderman <ben@happyspork.com>
 Part of Sporktris http://www.happyspork.com/games
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY.
 
 See the License.txt file for more details.
 */

#include "highscores.h"
#include "namedv.h"
#include "prefs_file.h"
#include "common_funcs.h"
#include "layers.h"
#include <string>
#include <fstream>
#include <iostream>
#include <SDL_image.h>
#include <SDL_ttf.h>

using namespace std;

highscores::highscores (Namedv *iPrefs)
{
	prefs = iPrefs;
	prefs_file::read((*prefs)["highscore_config"].c_str(), &config);
	Namedv nv;
	install_path = (*prefs)["install_prefix"]+config["hs_file"];
	prefs_file::read(install_path.c_str(), &nv);
	for (unsigned int i = 0; i < max_scores; i++)
	{
		string temp = string("hs")+funcs::str_int(i)+string("_");
		hs_entry temp_hs(nv[temp+"name"], nv.geti(temp+"score"), nv.geti(temp+"level"), nv.geti(temp+"lines"));
		if (temp_hs.name == "") temp_hs.name = "None";
		scores.push_back( temp_hs );
	}
	last_name = nv["last_name"];
}

void highscores::save_scores (void)
{
	Namedv nv;
	for (unsigned int i = 0; i < max_scores; i++)
	{
		string temp = string("hs")+funcs::str_int(i)+string("_");
		nv[temp+"score"] = funcs::str_int(scores[i].score);
		nv[temp+"level"] = funcs::str_int(scores[i].level);
		nv[temp+"lines"] = funcs::str_int(scores[i].lines);
		nv[temp+"name"] = scores[i].name;
	}
	nv["last_name"] = last_name;
	prefs_file::save(install_path.c_str(), &nv);
}

bool highscores::add_score (int score, int level, int lines)
{
	bool high = false;
	if (scores.size() < max_scores) high = true;
	for (unsigned int i = 0; i < scores.size(); i++)
	{
		if (score > scores[i].score) high = true;
	}
	if (!high) return false;
	
	SDL_Surface *screen = SDL_GetVideoSurface();
	layerset ls;
	SDL_Surface *background = funcs::load_surf(config["high_score_bg"], 0, "high score bg");
	SDL_Surface *input_box = funcs::load_surf(config["ibox"], 3, "high score input box");
	SDL_Surface *text_surf = funcs::load_surf(config.geti("input_w"), config.geti("input_h"), "high scores text");
	SDL_Surface *header = funcs::load_surf(config["header_img"], 3, "high scores header");
	ls.add(layer("background", background, 0, 0));
	ls.add(layer("header", header, screen->w/2-header->w/2, prefs->geti("header_y")));
	ls.add(layer("input_box", input_box, config.geti("ibox_x"), config.geti("ibox_y")));
	ls.add(layer("input_text", text_surf, config.geti("ibox_x")+config.geti("input_x"), config.geti("ibox_y")+config.geti("input_y")));
	string name = last_name;
	funcs::init_ttf();
	TTF_Font *font = TTF_OpenFont(config["font"].c_str(), config.geti("input_font_size"));
	if (font == NULL)
	{
		printf("couldn't load font from file \"%s\"\n", config["font"].c_str());
	}
	SDL_Color fcolor = funcs::make_color(config["input_font_color"]);
	SDL_FillRect(text_surf, NULL, SDL_MapRGBA(text_surf->format, 0, 0, 0, 0));
	blit_text(text_surf, font, name, fcolor);
	ls.blit(SDL_GetVideoSurface());
	SDL_Flip(SDL_GetVideoSurface());
	
	SDL_Event e;
	bool quitting = false;
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	while (SDL_WaitEvent(&e) != 0)
	{
		if (e.type == SDL_KEYDOWN)
		{
			SDL_keysym ks = e.key.keysym;
			if (ks.sym >= SDLK_a && ks.sym <= SDLK_z && name.size() < (unsigned)config.geti("max_name_length"))
			{
				char chr = ks.sym;
				if (ks.mod & KMOD_LSHIFT) chr -= 32;
				name += chr;
				ls.findByName("input_text")->changed = true;
			} else if (ks.sym == SDLK_RETURN && name != "") {
				break;
			} else if (ks.sym == SDLK_BACKSPACE && name.size() != 0) {
				name = name.substr(0, name.size()-1);
				ls.findByName("input_text")->changed = true;
			}
		} else if (e.type == SDL_QUIT) {
			quitting = true;
			break;
		}
		SDL_FillRect(text_surf, NULL, SDL_MapRGBA(text_surf->format, 0, 0, 0, 0));
		blit_text(text_surf, font, name, fcolor);
		ls.blit(screen);
		SDL_Flip(screen);
	}
	SDL_EnableKeyRepeat(0, 0);
	
	if (!quitting)
	{
		sort_scores();
		int insert_at = scores.size();
		for (int i = scores.size()-1; i >= 0; i--)
		{
			if (score > scores[i].score) insert_at = i;
		}
		scores.insert(scores.begin()+insert_at, hs_entry(name, score, level, lines));
		if (scores.size() > max_scores) scores.pop_back();
		last_name = name;
		save_scores();
		quitting = show_scores(insert_at);
	}
	
	SDL_FreeSurface(background);
	SDL_FreeSurface(header);
	SDL_FreeSurface(input_box);
	SDL_FreeSurface(text_surf);
	TTF_CloseFont(font);
	return quitting;
}

bool highscores::show_scores (int highlight)
{
	SDL_Surface *screen = SDL_GetVideoSurface();
	layerset ls;
	SDL_Surface *background = funcs::load_surf(config["high_score_bg"], 0, "high score bg");
	SDL_Surface *entry_bg = funcs::load_surf(config["entry_img"], 3, "high score entry bg");
	SDL_Surface *header = funcs::load_surf(config["header_img"], 3, "high scores header");
	ls.add(layer("background", background, 0, 0));
	ls.add(layer("header", header, screen->w/2-header->w/2, prefs->geti("header_y")));
	funcs::init_ttf();
	TTF_Font *font = TTF_OpenFont(config["font"].c_str(), config.geti("entry_font_size"));
	if (font == NULL)
	{
		printf("Couldn't load font from file \"%s\": %s\n", config["font"].c_str(), TTF_GetError());
	}
	SDL_Color color = funcs::make_color(config["entry_font_color"]);
	SDL_Color hcolor = funcs::make_color(config["entry_highlight_color"]);
	
	vector<SDL_Surface*> entries;
	//headers
	SDL_Surface *headers = funcs::load_surf(entry_bg->w, entry_bg->h, "highscore headers");
	SDL_FillRect(headers, NULL, SDL_MapRGBA(headers->format, 0, 0, 0, 0));
	blit_text(headers, font, "#", hcolor, config.geti("entry_number"));
	blit_text(headers, font, "Name", hcolor, config.geti("entry_name"));
	blit_text(headers, font, "Score", hcolor, config.geti("entry_score"), true);
	blit_text(headers, font, "Lines", hcolor, config.geti("entry_lines"), true);
	blit_text(headers, font, "Level", hcolor, config.geti("entry_level"), true);
	int hx = prefs->geti("window_w")/2-entry_bg->w/2;
	int hy = config.geti("entries_y")-entry_bg->h;
	ls.add(layer(string("headerbg"), entry_bg, hx, hy));
	ls.add(layer(string("headers"), headers, hx, hy));
	sort_scores();
	for (unsigned int i = 0; i < scores.size(); i++)
	{
		SDL_Color my_color = (unsigned)highlight == i ? hcolor : color;
		SDL_Surface *entry = funcs::load_surf(entry_bg->w, entry_bg->h, "highscore entry");
		SDL_FillRect(entry, NULL, SDL_MapRGBA(entry->format, 0, 0, 0, 0));
		blit_text(entry, font, funcs::str_int(i+1), my_color, config.geti("entry_number"));
		blit_text(entry, font, scores[i].name, my_color, config.geti("entry_name"));
		blit_text(entry, font, funcs::str_int(scores[i].score), my_color, config.geti("entry_score"), true);
		blit_text(entry, font, funcs::str_int(scores[i].lines), my_color, config.geti("entry_lines"), true);
		blit_text(entry, font, funcs::str_int(scores[i].level), my_color, config.geti("entry_level"), true);
		entries.push_back(entry);
		int x = prefs->geti("window_w")/2-entry_bg->w/2;
		int y = config.geti("entries_y")+entry_bg->h*i;
		ls.add(layer(string("entrybg")+funcs::str_int(i), entry_bg, x, y));
		ls.add(layer(string("entry")+funcs::str_int(i), entry, x, y));
	}
	
	ls.blit(screen);
	SDL_Flip(screen);
	
	TTF_CloseFont(font);
	SDL_FreeSurface(background);
	SDL_FreeSurface(entry_bg);
	SDL_FreeSurface(header);
	SDL_FreeSurface(headers);
	for (unsigned int i = 0; i < entries.size(); i++)
	{
		SDL_FreeSurface(entries[i]);
	}
	entries.clear();
	
	SDL_Event event;
	while (SDL_WaitEvent(&event) != 0)
	{
		if (event.type == SDL_QUIT)
		{
			return true;
		} else if (event.type == SDL_KEYDOWN) {
			SDLKey sym = event.key.keysym.sym;
			if (sym == SDLK_RETURN || sym == SDLK_ESCAPE || sym == SDLK_SPACE)
			{
				return false;
			}
		} else if (event.type == SDL_MOUSEBUTTONDOWN) {
			return false;
		}
	}
	
	return false;
}

void highscores::sort_scores (void)
{
	vector<hs_entry> sorted;
	for (unsigned int i = 0; i < scores.size(); i++)
	{
		vector<hs_entry>::iterator insert_at = sorted.end();
		if (sorted.size() != 0)
		{
			int highest = scores[i].score;
			for (int s = sorted.size()-1; s >= 0; s--)
			{
				if (highest > sorted[s].score)
				{
					insert_at = sorted.begin()+i;
					highest = sorted[s].score;
				}
			}
		}
		sorted.insert(insert_at, scores[i]);
	}
}

void highscores::blit_text (SDL_Surface *surf, TTF_Font *font, const string& text, SDL_Color color, int x, bool align_right)
{
	if (text != "")
	{
		SDL_Surface *temp = TTF_RenderText_Blended(font,  text.c_str(), color);
		SDL_Rect trect, drect;
		SDL_GetClipRect(temp, &trect);
		SDL_GetClipRect(surf, &drect);
		drect.y = drect.h/2-trect.h/2;
		drect.x = x;
		if (align_right) drect.x = drect.x-trect.w;
		SDL_SetAlpha(temp, 0, 255);
		SDL_BlitSurface(temp, &trect, surf, &drect);
		SDL_FreeSurface(temp);
	}
}

